%% program solves a portfolio problem for an agent whose preferences
%% exhibit both first-order risk aversion AND narrow framing
%% creates the top part of Figure 3

%% gross risk-free rate
rf = 1.02;
%% mean and standard deviation of the log stock market return
rm = 0.06;
rs = 0.2;
%% mean and standard deviation of the log return on the
%% non-financial asset
nm = 0.04;
ns = 0.03;
%% correlation between the log returns on the stock market and
%% the non-financial asset
omg = 0.1;
%% fixed fraction of wealth allocated to the non-financial asset
thn = 0.75;

%% parameters of the aggregator function W(.,.)
%% beta (bet) and rho (rho)
bet = 0.9;
rho = -1;
%% Figure 3 looks at a range of values of the preference parameters
%% gamma (gm) and lambda (lam)
gm_min = 1.1;
gm_max = 10;
lam_min = 1.1;
lam_max = 4;
%% degree of narrow framing
b_0 = 0.1;

%% fineness of grid discretizing the (gamma,lambda) plane
lim = 40;

%% fineness of grid used to discretize the area being integrated over
grd_fin = 51;
stp = 10/(grd_fin-1);
grd = [-5:stp:5]';
grd1 = kron(grd,ones(grd_fin,1));
grd2 = kron(ones(grd_fin,1),grd);

%% the weighting vector for Simpson's rule (double integration)
s1 = 4*kron(ones((grd_fin-3)/2,1),[1;0])+2*kron(ones((grd_fin-3)/2,1),[0;1]);
s1 = [1; s1; 4; 1];
s2 = kron(s1,ones(grd_fin,1));
s3 = kron(ones(grd_fin,1),s1);
s4 = s2.*s3;

%% array marking the range of (gamma,lambda) plane where the agent
%% does not participate in the stock market
dgn = zeros(lim,lim);

%% parameter used in the calculations
eps_s = (log(rf)-rm)/rs;

for count = 1:lim;
   count
   for count2 = 1:lim;
      gm = gm_min + (count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min + (count2-1)*(lam_max-lam_min)/(lim-1);
      
v2 = exp(rm+rs^2/2)*(erf((eps_s-rs)/sqrt(2))/2+0.5)-rf*(erf(eps_s/sqrt(2))/2+0.5);
v2 = v2*(lam-1) + exp(rm+rs^2/2) - rf;

aph_0 = 0.001;
aph_1 = 0.15;

%% compute function at first endpoint

A = (1-bet)^(1/rho)*aph_0^((rho-1)/rho);
t1 = [];

for ths = 0:0.01:(1-thn);
   u1 = (1-ths-thn)*rf+thn*exp(nm+ns*grd1)+ths*exp(rm+rs*grd2);
   u1 = (u1.^(1-gm)).*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
   u1 = (u1'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
   u1 = u1^(1/(1-gm));
   w1 = A*u1+b_0*ths*v2;
   t1 = [t1;w1];
end;
[q1 q2] = max(t1);
f_0 = 1+((1-bet)/bet)^(1/(rho-1))*q1^(-rho/(rho-1));
f_0 = 1/f_0 - aph_0;

%% compute function at second endpoint

A = (1-bet)^(1/rho)*aph_1^((rho-1)/rho);
t1 = [];
for ths = 0:0.01:(1-thn);
   u1 = (1-ths-thn)*rf+thn*exp(nm+ns*grd1)+ths*exp(rm+rs*grd2);
   u1 = (u1.^(1-gm)).*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
   u1 = (u1'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
   u1 = u1^(1/(1-gm));
   w1 = A*u1+b_0*ths*v2;
   t1 = [t1;w1];
end;
[q1 q2] = max(t1);
f_1 = 1+((1-bet)/bet)^(1/(rho-1))*q1^(-rho/(rho-1));
f_1 = 1/f_1 - aph_1;

%% now use interval bisection

flg = 1;
ct = 1;

while flg > 0.000001;
   ct = ct + 1;
   aph_2 = (aph_0 + aph_1)/2;
   flg = abs(aph_2-aph_1);
   A = (1-bet)^(1/rho)*aph_2^((rho-1)/rho);
   t1 = [];
   for ths = 0:0.01:(1-thn);
      u1 = (1-ths-thn)*rf+thn*exp(nm+ns*grd1)+ths*exp(rm+rs*grd2);
      u1 = (u1.^(1-gm)).*exp(-(grd1.^2-2*omg*grd1.*grd2+grd2.^2)/(2*(1-omg^2)));
      u1 = (u1'*s4)*stp^2/(18*pi*sqrt(1-omg^2));
      u1 = u1^(1/(1-gm));
      w1 = A*u1+b_0*ths*v2;
      t1 = [t1;w1];
   end;
   [q1 q2] = max(t1);
   f_2 = 1+((1-bet)/bet)^(1/(rho-1))*q1^(-rho/(rho-1));
   f_2 = 1/f_2 - aph_2;
   if sign(f_1) == sign(f_2);
      aph_1 = aph_2;
      f_1 = f_2;
   else;
      aph_0 = aph_1;
      aph_1 = aph_2;
      f_0 = f_1;
      f_1 = f_2;
   end;
end;

aph = aph_1;
ths_max = (q2-1)*0.01;

if ths_max == 0;
   dgn(count,count2) = 1;
end;

end;
end;

plt = [];

for count = 1:lim;
   for count2 = 1:lim;
      gm = gm_min+(count-1)*(gm_max-gm_min)/(lim-1);
      lam = lam_min+(count2-1)*(lam_max-lam_min)/(lim-1);
      if dgn(count,count2) == 1;
         plt = [plt; gm lam];
      end;
   end;
end;

figure;

subplot(2,1,1);
plot(plt(:,1),plt(:,2),'+');
axis([gm_min gm_max lam_min lam_max]);
xlabel('\gamma');
ylabel('\lambda');
